`physioset_import` - Import physioset from disk file
===

The `physioset_import` node creates a physioset from a disk file, using a
user-defined data importer. The `physioset_import` is special in the sense that,
contrary to all other nodes, it does not expect a `physioset` object as input.

## Usage synopsis:

Create a physioset object from a numeric matrix:

````matlab
import meegpipe.node.*;

myImporter = physioset.import.matrix;
myNode = physioset_import.new('Importer', myImporter);
data = run(myNode, randn(10, 1000));
````
In the example above, the input to method `run()` is a built-in numeric data
matrix because that is the type of input expected by method `import()` of the
[physioset.import.matrix][matrix-importer] importer. However, the
`physioset_import` node is typically used to import disk files. Below an example
with an `.edf` file:

````matlab
myImporter = physioset.import.edfplus;
myNode = physioset_import.new('Importer', myImporter);
data = run(myNode, '/path/to/myfile.edf');
````

[matrix-importer]: ../../../+physioset/+import/+matrix/README.md


## Construction arguments

The `physioset_import` node admits all the key/value pairs admitted by the
[abstract_node][abstract-node] class. For keys specific to this node
class see the documentation of the helper [config][config] class.

[abstract-node]: ../@abstract_node/README.md
[config]: ./config.md


## Methods

See the documentation of the [node API documentation][node].

[node]: ../



## Usage examples


### Import multiple files

````matlab
import meegpipe.node.*;
myImporter  = physioset.import.mff;
myNode      = physioset_import.new('Importer', myImporter);

myFiles = {...
    '/path/to/onefile.mff', ...
    '/path/to/anotherfile.mff', ...
    '/path/to/yetanotherfile.mff' ...
    };

dataArray   = run(myNode, myFiles{:});
`````
Where `dataArray` is a `1x3` cell array of `physioset` objects.


### Sensors specifications

Some file formats do not contain detailed information regarding the experimental
sensors that were used to obtain the data. For instance, the [EDF/EDF+][edf]
format does not store the coordinates of the EEG sensors. In this case, you may
want to specify the measurement sensors when performing the data import so that
the generated `physioset` is as complete as possible.

Assume that file `/path/to/file.edf` has been recorded using an [EGI][egi]
HydroCel GSN 256 sensor net. The following code snippet will generate
a `physioset` object with appropriate sensor coordinates:

````matlab
import meegpipe.node.*;
mySensors  = sensors.eeg.from_template('egi256');
myImporter = physioset.import.edf('Sensors', mySensors);
myNode     = physioset_import.new('Importer', myImporter);
data       = run(myNode, '/path/to/file.edf');
````

[egi]: http://www.egi.com


### Import in single precision

````matlab
myImporter = physioset.import.edf('Precision', 'single');
myNode     = physioset_import.new('Importer', myImporter);
data       = run(myNode, '/path/to/file.edf');
````

### Minimize virtual memory usage

The `physioset_import` can produce a `physioset` object that will use as
little virtual memory as possible, at the expense of slower read/write
performance. The following code snippet illustrates a prototypical example of
importing a very large `.mff` file, selecting a subset of data channels, and
downsampling it to make it more manageable:

````matlab
% We will implement the process as a multi-node pipeline
nodeList = {};

% Use single precision, and release allocated VM as soon as possible
myImporter = physioset.import.mff(...
    'Precision', 'single', 'AutoDestroyMemMap', true);
myNode   = physioset_import.new('Importer', myImporter);
nodeList = [nodeList; {myNode}];

% subset nodes will create a new (smaller) physioset object
% This node in particular will select channels with indices 4:20
mySel = pset.selector.sensor_idx(4:20);
myNode = subset.new('SubsetSelector', mySel, 'AutoDestroyMemMap', true);
nodeList = [nodeList; {myNode}];

% A downsampling node
% We don't set the AutoDestroyMemMap option to true this time because the
% physioset generated by this node is probably small enough. Using the default
% (false) value for AutoDestroyMemMap will speed up subsequent steps performed
% on the output of the resample node.
myNode = resample.new('DownsampleBy', 4);
nodeList = [nodeList; {myNode}];


myPipe = pipeline.new('NodeList', nodeList);
data = run(myPipe, '/path/to/mfffile.mff');
````



