Splitting a sleep recording into sleep stages
===

This tutorials illustrates how _meegpipe_ can be used to split a whole night
sleep recording into a set of files that contain only data from the same 
sleep stage. That is, we have a set of large files (in `.pset/.pseth` 
format) like:

````
ssmd_0104_eeg_sleep_1.pset
ssmd_0104_eeg_sleep_1.pseth
...
ssmd_0108_eeg_sleep_1.pset
ssmd_0108_eeg_sleep_1.pseth
```` 

And, for each pair of `.pset/.pseth` files we want to produce smaller files
that contain only data from a given sleep stage, i.e. something like:

````
ssmd_0104_eeg_sleep_1_wakefulness.pset
ssmd_0104_eeg_sleep_1_wakefulness.pseth
ssmd_0104_eeg_sleep_1_nrem1.pset
ssmd_0104_eeg_sleep_1_nrem1.pseth
...
````


## Preliminaries

Before anything else you need to ensure that EEGLAB is in your MATLAB 
search path. You also need to initialize _meegpipe_:

````
addpath(genpath('/data1/toolbox/eeglab'));
meegpipe.initialize;
````

## Retrieve sleep data and sleep scores

I assume that you are working at `somerengrid` (our lab's private computing
grid) and that the relevant sleep data files are managed by the
 [somsds][somsds] data management system. Thus we can retrieve the 
relevant data files for recording `ssmd` and subjects `104` and `108` as
 follows:

[somsds]: http://www.germangh.com/somsds/

````matlab
% Create symbolic links to the relevant sleep recordings
files = somsds.link2rec('ssmd', ...
    'subject',      [104 108], ...
    'modality',     'eeg', ...
    'condition',    'sleep', ...
    'session',      1, ...         % Only the first night data
    'file_regex',   'pset.?$' ...  % Only files in pset/pseth format
);
````

The command above will generate symbolic links with names like:

````
ssmd_0104_eeg_sleep_1.pset
ssmd_0104_eeg_sleep_1.pseth
...
ssmd_0105_eeg_sleep_1.pset
ssmd_0105_eeg_sleep_1.pseth
````

You will need to manually place the `.mat` files with the sleep scores (as
produced by Giovanni Piantoni's [sleep scoring toolbox][sctoolbox]) within
 the same directory where the links above are located. Also the names of 
sleep scores `.mat` files must follow the naming convention illustrated 
below:

[sctoolbox]: https://github.com/gpiantoni/sleepscoring

````
ssmd_0104_eeg_sleep_1.pset
ssmd_0104_eeg_sleep_1.pseth
ssmd_0104_eeg_scores_sleep_1.mat
...
ssmd_0105_eeg_sleep_1.pset
ssmd_0105_eeg_sleep_1.pseth
ssmd_0105_eeg_scores_sleep_1.mat
````


## Building the pipeline

The following code snippet will build the pipeline that you need to 
extract the subset of your data that has been scored as _NREM 1_. 
Similar pipelines could be built to extract other sleep stages.

````matlab

nodeList = {};

% Node 1: Load the physioset
myNode = meegpipe.node.physioset_import.new('Importer', physioset.import.physioset);
nodeList = [nodeList {myNode}];

% Node 2: Embed sleep scores in physioset as events
myNode = meegpipe.node.ev_gen.new(...
    'EventGenerator', physioset.event.sleep_scores_generator);
nodeList = [nodeList {myNode}];

% Node 3: Extract all NREM 1 data

evSel1 = physioset.event.class_selector('Type', 'NREM 1');
evSel2 = physioset.event.property_match_selector('Scorer', 'Jennifer');
evSel = physioset.event.cascade_selector(evSel1, evSel2);
dataSel = pset.selector.event_selector(evSel);
myNode = meegpipe.node.subset.new('DataSelector', dataSel);

nodeList = [nodeList {myNode}];

% Build the pipeline
myPipe = meegpipe.node.pipeline.new('NodeList', nodeList, ...
    'Name',           'nrem1', ...
    'GenerateReport', false, ...
    'OGE',            false, ...
    'Queue',          'short.q@somerenserver.herseninstituut.knaw.nl', ...
    'Save',           true ...
);

````

Note that we used only the scores produced by `Jennifer` to mark the
 locations of NREM 1 epochs. Typically, a sleep recording will be scored 
by multiple scorers that will agree only partially. At this point, 
_meegpipe_ does not implement any smart way of fusing scores from multipe 
scorers. That is why we need to specify the scorer that should be used
to extract the various sleep stages.


## Running the pipeline


````matlab
% Recall that files is the cell array generated by somsds.link2rec above
% Thus inputPath is the directory where the links were generaetd
inputPath = fileparts(files{1});

% Pick only the pseth files
files = misc.dir(inputPath, 'pseth$', false, false, true);
run(myPipe, files{:});
````


## Where are the converted files?

As usual, _meegpipe_ stores the processing results for file `fileX.pseth` 
under a directory called `fileX.meegpipe`. Under Linux or Mac OS X you can
 use the shell utility `find` to locate the files you want. For instance, 
you could move all _nrem1_ and _rem_ data splits to directory 
`/data/splits` using the following shell commands:

````
find ./ -regex '.*_(nrem1|rem)\.pset.*' | xargs -I{} cp "{}" /data/splits
```` 
